#region Using declarations
using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
//using System.Windows.Media.FontFamily;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
//using NinjaTrader.NinjaScript.Indicators;
using NinjaTrader.NinjaScript.DrawingTools;

#endregion

namespace NinjaTrader.NinjaScript.Indicators
{
	/// <summary>
	/// 01/06/2018 - Ported to NT8
	/// 04/14/2016 - Display correctly values with large fractions that C# converts to scientific notation.
	/// 02/27/2015 - Added Current price marker.  Also, uses much less resources with 
	/// CalculateOnBarClose = false.
	/// </summary>
    [Description("Unirenko Projected Bar Closes")]
    public class jprUniRenkoBarCloses : Indicator
	{
        #region Variables
			int    trendTicks=0, reversalTicks=0, tickOffset=0;
			double trendValue=0, reversalValue=0, tickOffsetValue=0;
			double nextBarUp=0,  nextBarDn=0;
			int    lastBar=0, barx=0;
			SimpleFont smallFont;
			string 					priceFormat 		= "0.00";
			private int 			lengthDisp 			= 150;

        #endregion


        protected override void OnBarUpdate() 
		{
			Print(Time[0]+"   OnBU: currB="+CurrentBar+"  lastB="+lastBar+"  trendTicks="+trendTicks+"  close="+Close[0]+"  nextBd="+nextBarDn+"  nextBu="+nextBarUp);
			if ( CurrentBar != lastBar && trendTicks > 0 ) 
			{
				if ( (Close[barx] > Open[barx]) ) 
				{
					nextBarUp = Close[barx] + trendValue + (tickOffsetValue);
					nextBarDn = Close[barx] - reversalValue - (tickOffsetValue );
				}
				else 
				{
					nextBarUp = Close[barx] + reversalValue + (tickOffsetValue );
					nextBarDn = Close[barx] - trendValue - (tickOffsetValue );
				}

				
	  	 		Draw.Text(this, "nxtBarUp", false, "    ---- " +nextBarUp.ToString(priceFormat), 0, nextBarUp, 0, TextColor, smallFont, TextAlignment.Left, Brushes.Transparent, Brushes.Transparent, 0);
	    		Draw.Text(this, "nxtBarDn", false, "    ---- " +nextBarDn.ToString(priceFormat), 0, nextBarDn, 0, TextColor, smallFont, TextAlignment.Left, Brushes.Transparent, Brushes.Transparent, 0);
				if (ShowMidProjPrice)
	  	 			Draw.Text(this, "nxtBarMid", false, "    ---- " +nextBarUp.ToString(priceFormat), 0, ((nextBarDn+nextBarUp)/2), 0, TextColor, smallFont, TextAlignment.Left, Brushes.Transparent, Brushes.Transparent, 0);
			}
			lastBar = CurrentBar;
			
			RemoveDrawObject("Current");
			if (Close[0] != nextBarUp && Close[0] != nextBarDn)
			{
	    		Draw.Text(this, "Current", false, "    ---- " +Close[0].ToString(priceFormat), 0, Close[0], 0, TextColorCurrent, smallFont, TextAlignment.Left, Brushes.Transparent, Brushes.Transparent, 0);
			}
			
			if (ShowPriceLine)
        		Draw.Line(this, "Current_price", true, Math.Min(CurrentBar, lengthDisp), Close[0], 0, Close[0], TextColorCurrent, DashStyleHelper.Dot, 1);
		}
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
	            IsOverlay				= true;
				
				ShowPriceLine			= true;
				TickOffset				= 0;
				TextColor				= Brushes.Blue;
				TextColorCurrent		= Brushes.DeepSkyBlue;
				PriceFontZize			= 8;
				Calculate 				= Calculate.OnPriceChange;
			}
			else if (State == State.Configure)
			{
				Match m;
				barx = (Calculate == Calculate.OnBarClose)? 0 : 1;	
				if ((m = Regex.Match(Bars.ToString(), @"UniR.*?T(\d+)R(\d+)", RegexOptions.IgnoreCase)).Success ) {
					Int32.TryParse(m.Groups[1].Value, out trendTicks);
					Int32.TryParse(m.Groups[2].Value, out reversalTicks);
				}
				decimal tickSizeDec = (decimal)(TickSize);
				reversalValue = reversalTicks * TickSize;
				trendValue = trendTicks * TickSize;
				tickOffsetValue = TickOffset * TickSize;
				priceFormat = Regex.Replace(tickSizeDec.ToString(),"([\\d])","0");
				priceFormat = Regex.Replace(priceFormat,",", ".");
				smallFont = new SimpleFont("Courier", PriceFontZize);

			}
		}
	   


		#region Properties
//		[Description("Display Horizontal Line on Current Price.")]
		[Display(ResourceType = typeof(Custom.Resource), Name = "ShowLineAtCurrentPrice", GroupName = "NinjaScriptParameters", Order = 0)]
		public bool ShowPriceLine
		{ get; set; }
		
//		[Description("Display Price Mid Projected Level.")]
		[Display(ResourceType = typeof(Custom.Resource), Name = "ShowMidProjectedPrice", GroupName = "NinjaScriptParameters", Order = 1)]
		public bool ShowMidProjPrice
		{ get; set; }
		
//		[Description("Offset Potential Close markers by this number of positive/negative ticks.")]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Proj Marker Tick Offset", GroupName = "NinjaScriptParameters", Order = 2)]
		public int TickOffset 
			{ get; set; }
			
//		[Description("Projected marker font size.")]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Proj Marker Font Size", GroupName = "NinjaScriptParameters", Order = 3)]
		public int PriceFontZize 
			{ get; set; }
			
//      [Description("Marker Text Color")]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Marker Text Color", GroupName = "NinjaScriptParameters", Order = 4)]
        public Brush TextColor
			{ get; set; }
        [Browsable(false)]
        public string TextColorSerialize
        {
            get { return Serialize.BrushToString(this.TextColor); }
            set { this.TextColor = Serialize.StringToBrush(value); }
        }
		
//      [Description("Current Price Text Color")]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Current Price Text Color", GroupName = "NinjaScriptParameters", Order = 5)]
        public Brush TextColorCurrent
			{ get; set; }
        [Browsable(false)]
        public string TextColorCurrentSerialize
        {
            get { return Serialize.BrushToString(this.TextColorCurrent); }
            set { this.TextColorCurrent = Serialize.StringToBrush(value); }
        }		
			
			
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private jprUniRenkoBarCloses[] cachejprUniRenkoBarCloses;
		public jprUniRenkoBarCloses jprUniRenkoBarCloses()
		{
			return jprUniRenkoBarCloses(Input);
		}

		public jprUniRenkoBarCloses jprUniRenkoBarCloses(ISeries<double> input)
		{
			if (cachejprUniRenkoBarCloses != null)
				for (int idx = 0; idx < cachejprUniRenkoBarCloses.Length; idx++)
					if (cachejprUniRenkoBarCloses[idx] != null &&  cachejprUniRenkoBarCloses[idx].EqualsInput(input))
						return cachejprUniRenkoBarCloses[idx];
			return CacheIndicator<jprUniRenkoBarCloses>(new jprUniRenkoBarCloses(), input, ref cachejprUniRenkoBarCloses);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.jprUniRenkoBarCloses jprUniRenkoBarCloses()
		{
			return indicator.jprUniRenkoBarCloses(Input);
		}

		public Indicators.jprUniRenkoBarCloses jprUniRenkoBarCloses(ISeries<double> input )
		{
			return indicator.jprUniRenkoBarCloses(input);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.jprUniRenkoBarCloses jprUniRenkoBarCloses()
		{
			return indicator.jprUniRenkoBarCloses(Input);
		}

		public Indicators.jprUniRenkoBarCloses jprUniRenkoBarCloses(ISeries<double> input )
		{
			return indicator.jprUniRenkoBarCloses(input);
		}
	}
}

#endregion
